@@ -13,19 +13,28 @@
 #include <kandroidextras/androidtypes.h>
 #include <kandroidextras/uri.h>
 
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 #include <QtAndroid>
+#else
+#include <QCoreApplication>
+#endif
 
 JNI_TYPE(org, kde, itinerary, Activity)
 
 /** Interface to the Java Activity class. */
 class ItineraryActivity {
-    JNI_OBJECT(ItineraryActivity, org::kde::itinerary::Activity)
+    JNI_UNMANAGED_OBJECT(ItineraryActivity, org::kde::itinerary::Activity)
 public:
-    JNI_METHOD(void, checkCalendar)
     JNI_METHOD(KAndroidExtras::Jni::Array<KAndroidExtras::java::lang::String>, attachmentsForIntent, KAndroidExtras::android::content::Intent)
-    JNI_METHOD(KAndroidExtras::android::net::Uri, openDocument, java::lang::String)
+    JNI_METHOD(KAndroidExtras::android::net::Uri, openDocument, KAndroidExtras::java::lang::String)
 private:
-    inline QAndroidJniObject handle() const { return QtAndroid::androidActivity(); }
+    inline QAndroidJniObject jniHandle() const {
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+        return QtAndroid::androidActivity();
+#else
+        return QNativeInterface::QAndroidApplication::context();
+#endif
+    }
 };
 
 #endif
