@@ -9,7 +9,6 @@
 #include <KItinerary/JsonLdDocument>
 #include <KItinerary/Place>
 
-#include <kcontacts_version.h>
 #include <KContacts/Address>
 
 #include <KCountry>
@@ -26,9 +25,6 @@
 #include <kandroidextras/jnisignature.h>
 #include <kandroidextras/javalocale.h>
 
-#include <QtAndroid>
-#include <QAndroidJniObject>
-
 using namespace KAndroidExtras;
 #endif
 
@@ -70,11 +66,7 @@ QString Localizer::formatAddress(const QVariant &obj) const
         return {};
     }
 
-#if KContacts_VERSION < QT_VERSION_CHECK(5, 92, 0)
-    return address.formattedAddress().replace(QLatin1String("\n\n"), QLatin1String("\n")).trimmed();
-#else
     return address.formatted(KContacts::AddressFormatStyle::MultiLineInternational);
-#endif
 }
 
 static bool needsTimeZone(const QDateTime &dt)
@@ -148,6 +140,21 @@ QString Localizer::formatDateTime(const QVariant& obj, const QString& propertyNa
     return s;
 }
 
+QString Localizer::formatDateOrDateTimeLocal(const QVariant& obj, const QString& propertyName) const
+{
+    const auto dt = JsonLdDocument::readProperty(obj, propertyName.toUtf8().constData()).toDateTime();
+    if (!dt.isValid()) {
+        return {};
+    }
+
+    // detect likely date-only values
+    if (dt.timeSpec() == Qt::LocalTime && (dt.time() == QTime{0, 0, 0} || dt.time() == QTime{23, 59, 59})) {
+        return QLocale().toString(dt.date(), QLocale::ShortFormat);
+    }
+
+    return QLocale().toString(dt.toLocalTime(), QLocale::ShortFormat);
+}
+
 QString Localizer::formatDuration(int seconds) const
 {
     if (seconds < 0) {
