@@ -25,9 +25,8 @@ public:
 
     QVector<QString> passes() const;
 
-    KPkPass::Pass* pass(const QString &passId);
+    Q_INVOKABLE KPkPass::Pass* pass(const QString &passId);
     Q_INVOKABLE bool hasPass(const QString &passId) const;
-    Q_INVOKABLE QObject* passObject(const QString &passId);
     Q_INVOKABLE QString passId(const QVariant &reservation) const;
 
     /** Import pass from a local @p url, returns the pass id if successful. */
