@@ -9,6 +9,7 @@
 #include "logging.h"
 
 #include <KItinerary/BusTrip>
+#include <KItinerary/ExtractorPostprocessor>
 #include <KItinerary/Reservation>
 #include <KItinerary/TrainTrip>
 #include <KItinerary/Ticket>
@@ -63,15 +64,15 @@ KPublicTransport::Location PublicTransport::locationFromPlace(const QVariant& pl
 
     if (JsonLd::isA<FlightReservation>(reservation) || JsonLd::isA<TrainReservation>(reservation) || JsonLd::isA<BusReservation>(reservation)) {
         loc.setName(KItinerary::LocationUtil::name(place));
-    } else {
-        const auto addr = KItinerary::LocationUtil::address(place);
-        loc.setStreetAddress(addr.streetAddress());
-        loc.setPostalCode(addr.postalCode());
-        loc.setLocality(addr.addressLocality());
-        loc.setRegion(addr.addressRegion());
-        loc.setCountry(addr.addressCountry());
     }
 
+    const auto addr = KItinerary::LocationUtil::address(place);
+    loc.setStreetAddress(addr.streetAddress());
+    loc.setPostalCode(addr.postalCode());
+    loc.setLocality(addr.addressLocality());
+    loc.setRegion(addr.addressRegion());
+    loc.setCountry(addr.addressCountry());
+
     const auto geo = KItinerary::LocationUtil::geo(place);
     loc.setCoordinate(geo.latitude(), geo.longitude());
 
@@ -254,14 +255,25 @@ static KItinerary::BusReservation applyJourneySection(KItinerary::BusReservation
     return res;
 }
 
+static QVariant postProcessOne(const QVariant &res)
+{
+    KItinerary::ExtractorPostprocessor postProc;
+    postProc.process({res});
+    const auto result = postProc.result();
+    if (result.size() == 1) {
+        return result.at(0);
+    }
+    return res;
+}
+
 QVariant PublicTransport::reservationFromJourneySection(const KPublicTransport::JourneySection &section)
 {
     using namespace KItinerary;
     if (isTrainMode(section.route().line().mode())) {
-        return ::applyJourneySection(TrainReservation(), section);
+        return postProcessOne(::applyJourneySection(TrainReservation(), section));
     }
     if (isBusMode(section.route().line().mode())) {
-        return ::applyJourneySection(BusReservation(), section);
+        return postProcessOne(::applyJourneySection(BusReservation(), section));
     }
 
     qCWarning(Log) << "Unsupported section type:" << section.route().line().mode();
@@ -273,10 +285,10 @@ QVariant PublicTransport::applyJourneySection(const QVariant &res, const KPublic
     using namespace KItinerary;
 
     if (JsonLd::isA<TrainReservation>(res)) {
-        return ::applyJourneySection(res.value<TrainReservation>(), section);
+        return postProcessOne(::applyJourneySection(res.value<TrainReservation>(), section));
     }
     if (JsonLd::isA<BusReservation>(res)) {
-        return ::applyJourneySection(res.value<BusReservation>(), section);
+        return postProcessOne(::applyJourneySection(res.value<BusReservation>(), section));
     }
 
     qCWarning(Log) << res.typeName() << "Unsupported section type!";
@@ -550,4 +562,9 @@ KOSMIndoorMap::Platform::Mode PublicTransport::lineModeToPlatformMode(KPublicTra
     return KOSMIndoorMap::Platform::Unknown;
 }
 
+KPublicTransport::Location PublicTransport::copyLocation(const KPublicTransport::Location &loc)
+{
+    return loc;
+}
+
 #include "moc_publictransport.cpp"
