@@ -5,6 +5,7 @@
 */
 
 #include "transfermanager.h"
+#include "constants.h"
 #include "logging.h"
 #include "favoritelocationmodel.h"
 #include "livedatamanager.h"
@@ -18,6 +19,7 @@
 #include <KItinerary/Flight>
 #include <KItinerary/LocationUtil>
 #include <KItinerary/Reservation>
+#include <KItinerary/SortUtil>
 #include <KItinerary/TrainTrip>
 
 #include <KPublicTransport/Journey>
@@ -252,6 +254,25 @@ void TransferManager::checkReservation(const QString &resId, const QVariant &res
     }
 }
 
+/** Checks whether @p loc1 and @p loc2 are far enough apart to need a tranfer,
+ *  with sufficient certainty.
+ */
+static bool isLikelyNotSameLocation(const QVariant &loc1, const QVariant &loc2)
+{
+    // if we are sure they are the same location we are done here
+    if (LocationUtil::isSameLocation(loc1, loc2, LocationUtil::WalkingDistance)) {
+        return false;
+    }
+
+    // if both have a geo coordinate we are also sure about both being different from the above check
+    if (LocationUtil::geo(loc1).isValid() && LocationUtil::geo(loc2).isValid()) {
+        return true;
+    }
+
+    // if we have to rely on the name, only do that if we are really sure they are different
+    return !LocationUtil::isSameLocation(loc1, loc2, LocationUtil::CityLevel);
+}
+
 TransferManager::CheckTransferResult TransferManager::checkTransferBefore(const QString &resId, const QVariant &res, Transfer &transfer) const
 {
     transfer.setAnchorTime(anchorTimeBefore(resId, res));
@@ -271,6 +292,9 @@ TransferManager::CheckTransferResult TransferManager::checkTransferBefore(const
     // - res is an event and we are not at its location already
     // ... and can happen in the following cases:
     // - res is not in a trip group at all (that assumes we are at home)
+    // - res is a location change, and the previous element is also a location change but not a connection
+    //   (ie. transfer from favorite location at the destination of a roundtrip trip group)
+
 
     const auto notInGroup = isNotInTripGroup(resId);
     if ((isLocationChange && isFirstInTripGroup(resId)) || notInGroup) {
@@ -305,13 +329,23 @@ TransferManager::CheckTransferResult TransferManager::checkTransferBefore(const
     } else {
         prevLoc = LocationUtil::location(prevRes);
     }
-    if (!toLoc.isNull() && !prevLoc.isNull() && !LocationUtil::isSameLocation(toLoc, prevLoc, LocationUtil::WalkingDistance)) {
+    if (!toLoc.isNull() && !prevLoc.isNull() && isLikelyNotSameLocation(toLoc, prevLoc)) {
         qDebug() << res << prevRes << LocationUtil::name(toLoc) << LocationUtil::name(prevLoc) << transfer.anchorTime();
         transfer.setFrom(PublicTransport::locationFromPlace(prevLoc, prevRes));
         transfer.setFromName(LocationUtil::name(prevLoc));
         transfer.setFloatingLocationType(Transfer::Reservation);
         return isLocationChange ? ShouldAutoAdd : CanAddManually;
+    }
 
+    // transfer to favorite at destination of a roundtrip trip group
+    if (LocationUtil::isLocationChange(res) && LocationUtil::isLocationChange(prevRes) && LocationUtil::isSameLocation(toLoc, prevLoc)) {
+        const auto arrivalTime = SortUtil::endDateTime(prevRes);
+        const auto departureTime = SortUtil::startDateTime(res);
+        transfer.setFloatingLocationType(Transfer::FavoriteLocation);
+        const auto f = pickFavorite(toLoc, resId, Transfer::Before);
+        transfer.setFrom(locationFromFavorite(f));
+        transfer.setFromName(f.name());
+        return std::chrono::seconds(arrivalTime.secsTo(departureTime)) < Constants::MaximumLayoverTime ? ShouldRemove : CanAddManually;
     }
 
     return ShouldRemove;
@@ -336,6 +370,8 @@ TransferManager::CheckTransferResult TransferManager::checkTransferAfter(const Q
     // - res is an event and the following or enclosing element is a lodging element
     // ... and can happen in the following cases
     // - res is not in a trip group at all (that assumes we are at home)
+    // - res is a location change, and the subsequent element is also a location change but not a connection
+    //   (ie. transfer to favorite location at the destination of a roundtrip trip group)
 
     const auto notInGroup = isNotInTripGroup(resId);
     if ((isLocationChange && isLastInTripGroup(resId)) || notInGroup) {
@@ -370,7 +406,7 @@ TransferManager::CheckTransferResult TransferManager::checkTransferAfter(const Q
     } else {
         nextLoc = LocationUtil::location(nextRes);
     }
-    if (!fromLoc.isNull() && !nextLoc.isNull() && !LocationUtil::isSameLocation(fromLoc, nextLoc, LocationUtil::WalkingDistance)) {
+    if (!fromLoc.isNull() && !nextLoc.isNull() && isLikelyNotSameLocation(fromLoc, nextLoc)) {
         qDebug() << res << nextRes << LocationUtil::name(fromLoc) << LocationUtil::name(nextLoc) << transfer.anchorTime();
         transfer.setTo(PublicTransport::locationFromPlace(nextLoc, nextRes));
         transfer.setToName(LocationUtil::name(nextLoc));
@@ -378,6 +414,17 @@ TransferManager::CheckTransferResult TransferManager::checkTransferAfter(const Q
         return isLocationChange ? ShouldAutoAdd : CanAddManually;
     }
 
+    // transfer to favorite at destination of a roundtrip trip group
+    if (LocationUtil::isLocationChange(res) && LocationUtil::isLocationChange(nextRes) && LocationUtil::isSameLocation(fromLoc, nextLoc)) {
+        const auto arrivalTime = SortUtil::endDateTime(res);
+        const auto departureTime = SortUtil::startDateTime(nextRes);
+        transfer.setFloatingLocationType(Transfer::FavoriteLocation);
+        const auto f = pickFavorite(fromLoc, resId, Transfer::After);
+        transfer.setTo(locationFromFavorite(f));
+        transfer.setToName(f.name());
+        return std::chrono::seconds(arrivalTime.secsTo(departureTime)) < Constants::MaximumLayoverTime ? ShouldRemove : CanAddManually;
+    }
+
     return ShouldRemove;
 }
 
@@ -417,17 +464,32 @@ bool TransferManager::isNotInTripGroup(const QString &resId) const
     return m_tgMgr->tripGroupIdForReservation(resId).isEmpty();
 }
 
+// default transfer anchor deltas (in minutes)
+enum { FlightDelta, TrainDelta, BusDelta, FallbackDelta };
+static constexpr const int default_deltas[][2] = {
+    { 90, 30 }, // Flight
+    { 20, 10 }, // Train
+    { 15, 10 }, // Bus
+    { 30, 15 }, // anything else
+};
+
 void TransferManager::determineAnchorDeltaDefault(Transfer &transfer, const QVariant &res) const
 {
     if (transfer.state() != Transfer::UndefinedState) {
         return;
     }
 
+    int delta;
     if (JsonLd::isA<FlightReservation>(res)) {
-        transfer.setAnchorTimeDelta(transfer.alignment() == Transfer::Before ? 60 * 60 : 30 * 60);
+        delta = default_deltas[FlightDelta][transfer.alignment()];
+    } else if (JsonLd::isA<TrainReservation>(res)) {
+        delta = default_deltas[TrainDelta][transfer.alignment()];
+    } else if (JsonLd::isA<BusReservation>(res)) {
+        delta = default_deltas[BusDelta][transfer.alignment()];
     } else {
-        transfer.setAnchorTimeDelta(10 * 60);
+        delta = default_deltas[FallbackDelta][transfer.alignment()];
     }
+    transfer.setAnchorTimeDelta(delta * 60);
 }
 
 QDateTime TransferManager::anchorTimeBefore(const QString &resId, const QVariant &res) const
